package com.streambase.contrib.currency.operator;

import java.net.URL;

import com.streambase.contrib.currency.graph.CurrencyGraph;
import com.streambase.contrib.currency.schemas.CurrencySchema;
import com.streambase.contrib.currency.schemas.SchemaValidator;
import com.streambase.sb.*;
import com.streambase.sb.operator.*;

/**
 * Generated by JDT StreamBase Client Templates (Version: 7.2.4.1203100759).
 *
 * This class is used as a Java Operator in a StreamBase application.
 * One instance will be created for each Java Operator in a StreamBase 
 * application. 
 * <p>
 * Enqueue methods should only be called from processTuple.
 * <p>
 * If there is any state that the operator instance needs to maintain beyond the
 * lifetime of the instance, the setSessionState/getSessionState methods can
 * be used.
 * @see Parameterizable
 * @see Operator
 * For in-depth information on implementing a custom Java Operator, please see
 * "Developing StreamBase Java Operators" in the StreamBase documentation.
 */
public class CurrencyArbitrage extends Operator implements Parameterizable {

	public static final long serialVersionUID = 1343156287078L;
	// Properties
	private double TransactionCosts;
	private String displayName = "Currency Arbitrage";
	// Local variables
	private int inputPorts = 2;
	private CurrencyGraph<String> cg;

	/**
	 * The constructor is called when the Operator instance is created, but before the Operator 
	 * is connected to the StreamBase application. We recommended that you set the initial input
	 * port and output port count in the constructor by calling setPortHints(inPortCount, outPortCount).
	 * The default is 1 input port, 1 output port. The constructor may also set default values for 
	 * operator parameters. These values will be displayed in StreamBase Studio when a new instance
	 * of this operator is  dragged to the canvas, and serve as the default values for omitted
	 * optional parameters.
	 */
	public CurrencyArbitrage() {
		super();
		setPortHints(2, 1);
		setDisplayName(displayName);
		setShortDisplayName(this.getClass().getSimpleName());
		setTransactionCosts(0.001D);
	}

	/**
	 * The typecheck method is called after the Operator instance is connected in the StreamBase
	 * application, allowing the Operator to validate its properties. The Operator class may 
	 * change the number of input or output ports by calling the requireInputPortCount(portCount)
	 * method or the setOutputSchema(schema, portNum) method. If the verifyInputPortCount method 
	 * is passed a different number of ports than the Operator currently has, a PortMismatchException
	 * (subtype of TypecheckException) is thrown.
	 */
	public void typecheck() throws TypecheckException {
		// typecheck: require a specific number of input ports
		requireInputPortCount(inputPorts);

		// Validate the input schemas and the operator properties
		if (getTransactionCosts() < 0 || getTransactionCosts() > 1) {throw new TypecheckException("The transaction costs contains an invalid number.");}
		SchemaValidator.validateSchema(getInputSchema(0), CurrencySchema.EDGE_WEIGHT_SCHEMA);
		SchemaValidator.validateSchema(getInputSchema(1), CurrencySchema.CONTROL_SCHEMA);
		
		// Set the output schema for the only output port
		setOutputSchema(0, CurrencySchema.ARBITRAGE_SCHEMA);
		
	}

	/**
	 * This method will be called by the StreamBase server for each Tuple given
	 * to the Operator to process. This is the only time an operator should 
	 * enqueue output Tuples.
	 * @param inputPort the input port that the tuple is from (ports are zero based)
	 * @param tuple the tuple from the given input port
	 * @throws StreamBaseException Terminates the application.
	 */
	public void processTuple(int inputPort, Tuple tuple)
			throws StreamBaseException {
		
		// If a tuple is received on the control port then process this
		if (inputPort > 0) {
			this.cg.setTransactionCost(tuple.getDouble("TransactionCost"));
			return;
		}
		
		// If a tuple is received on the currency port then update the appropriate edge 
		// in the graph and attempt to find an arbitrage opportunity to exploit. 
		Tuple out = CurrencySchema.ARBITRAGE_SCHEMA.createTuple();
		this.cg.setWeight(tuple.getString("Currency1"), tuple.getString("Currency2"), tuple.getDouble("Bid"));
		this.cg.setWeight(tuple.getString("Currency2"), tuple.getString("Currency1"), tuple.getDouble("Ask"));
		this.cg.calculateArbitrage();
		
		// If there is no arbitrage opportunity, do not output a tuple.
		if (!this.cg.isArbitrage()) {return;}				
		
		// If there is an arbitrage opportunity, output the currencies and the value of
		// the transaction (which includes transaction costs). Note that this transaction
		// is the arbitrage opportunity with the maximum profit. 
		out.setList("Currencies", this.cg.getArbSequence());
		out.setDouble("Value", this.cg.getProfit());
		sendOutput(0, out);
	}

	//Create a new instance of CurrencyGraph   
	public void init() throws StreamBaseException {
		super.init();
		this.cg = new CurrencyGraph<String>(getTransactionCosts());
	}
	

	 // The shutdown method is called when the StreamBase server is in the process of shutting down.
	public void shutdown() {
		System.out.println("Closing Arbitrage Adapter...");
	}
	
	/***************************************************************************************
	 * The getter and setter methods provided by the Parameterizable object.               *
	 * StreamBase Studio uses them to determine the name and type of each property         *
	 * and obviously, to set and get the property values.                                  *
	 ***************************************************************************************/

	public void setTransactionCosts(double TransactionCosts) {
		this.TransactionCosts = TransactionCosts;
	}

	public double getTransactionCosts() {
		return this.TransactionCosts;
	}


	@Override
	public URL getIconResource(IconKind iconType){
		switch(iconType){
			case CANVAS_OVERLAY_ICON:
				return getClass().getResource("dollar24.png");
			case PALETTE_ICON_LARGE:
				return getClass().getResource("dollar32.png");
			case PALETTE_ICON_SMALL:
				return getClass().getResource("dollar16.png");
		}
		return null;
	}

}
